# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceLineItemCategory = typing.Union[
    typing.Literal["EXPENSE", "REVENUE", "TAX", "SHIPPING", "DISCOUNT", "OTHER"], typing.Any
]
