# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class BankAddress(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.bank_lookup import BankAddress

    BankAddress(
        address="123 Main St",
        city="Anytown",
        state="CA",
        postal_code="12345",
        postal_code_extension="6789",
    )
    """

    address: str
    city: str
    state: str
    postal_code: str = pydantic.Field(alias="postalCode")
    postal_code_extension: str = pydantic.Field(alias="postalCodeExtension")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
