# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .email_sender_provider import EmailSenderProvider
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EmailSenderResponse(UniversalBaseModel):
    provider: EmailSenderProvider
    from_email: str = pydantic.Field(alias="fromEmail")
    from_name: str = pydantic.Field(alias="fromName")
    has_api_key: bool = pydantic.Field(alias="hasApiKey")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
