# This file was auto-generated by Fern from our API Definition.

import typing
from .invoice_creation_with_entity_request import InvoiceCreationWithEntityRequest
from .invoice_creation_with_entity_group_request import InvoiceCreationWithEntityGroupRequest

InvoiceCreationRequest = typing.Union[InvoiceCreationWithEntityRequest, InvoiceCreationWithEntityGroupRequest]
