# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from .business_onboarding_options import BusinessOnboardingOptions
from .individual_onboarding_options import IndividualOnboardingOptions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class OnboardingOptionsResponse(UniversalBaseModel):
    enable_business: bool = pydantic.Field(alias="enableBusiness")
    enable_individual: bool = pydantic.Field(alias="enableIndividual")
    payment_method: bool = pydantic.Field(alias="paymentMethod")
    business: BusinessOnboardingOptions
    individual: IndividualOnboardingOptions

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
