# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class PhoneNumber(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.commons import PhoneNumber

    PhoneNumber(
        country_code="1",
        number="4155551234",
    )
    """

    country_code: str = pydantic.Field(alias="countryCode")
    number: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
